@@ -32,16 +32,19 @@ import evdev
 import keysyms.keysymdef as _keysymdef
 import psutil
 
-from gi.repository import Gdk, GLib
-from solaar.ui.config_panel import change_setting as _change_setting
 from yaml import add_representer as _yaml_add_representer
 from yaml import dump_all as _yaml_dump_all
 from yaml import safe_load_all as _yaml_safe_load_all
 
 from .common import NamedInt
+from .device import Device as _Device
 from .hidpp20 import FEATURE as _F
 from .special_keys import CONTROL as _CONTROL
 
+import gi  # isort:skip
+gi.require_version('Gdk', '3.0')  # isort:skip
+from gi.repository import Gdk, GLib  # NOQA: E402 # isort:skip
+
 _log = getLogger(__name__)
 del getLogger
 
@@ -377,7 +380,7 @@ class RuleComponent:
 
 
 class Rule(RuleComponent):
-    def __init__(self, args, source=None):
+    def __init__(self, args, source=None, warn=True):
         self.components = [self.compile(a) for a in args]
         self.source = source
 
@@ -411,7 +414,7 @@ class Condition(RuleComponent):
 
 
 class Not(Condition):
-    def __init__(self, op):
+    def __init__(self, op, warn=True):
         if isinstance(op, list) and len(op) == 1:
             op = op[0]
         self.op = op
@@ -429,7 +432,7 @@ class Not(Condition):
 
 
 class Or(Condition):
-    def __init__(self, args):
+    def __init__(self, args, warn=True):
         self.components = [self.compile(a) for a in args]
 
     def __str__(self):
@@ -450,7 +453,7 @@ class Or(Condition):
 
 
 class And(Condition):
-    def __init__(self, args):
+    def __init__(self, args, warn=True):
         self.components = [self.compile(a) for a in args]
 
     def __str__(self):
@@ -503,12 +506,14 @@ def x11_pointer_prog():
 
 
 class Process(Condition):
-    def __init__(self, process):
+    def __init__(self, process, warn=True):
         self.process = process
         if wayland or not x11_setup():
-            _log.warn('rules can only access active process in X11 - %s', self)
+            if warn:
+                _log.warn('rules can only access active process in X11 - %s', self)
         if not isinstance(process, str):
-            _log.warn('rule Process argument not a string: %s', process)
+            if warn:
+                _log.warn('rule Process argument not a string: %s', process)
             self.process = str(process)
 
     def __str__(self):
@@ -526,12 +531,14 @@ class Process(Condition):
 
 
 class MouseProcess(Condition):
-    def __init__(self, process):
+    def __init__(self, process, warn=True):
         self.process = process
         if wayland or not x11_setup():
-            _log.warn('rules cannot access active mouse process in X11 - %s', self)
+            if warn:
+                _log.warn('rules cannot access active mouse process in X11 - %s', self)
         if not isinstance(process, str):
-            _log.warn('rule MouseProcess argument not a string: %s', process)
+            if warn:
+                _log.warn('rule MouseProcess argument not a string: %s', process)
             self.process = str(process)
 
     def __str__(self):
@@ -549,9 +556,10 @@ class MouseProcess(Condition):
 
 
 class Feature(Condition):
-    def __init__(self, feature):
+    def __init__(self, feature, warn=True):
         if not (isinstance(feature, str) and feature in _F):
-            _log.warn('rule Feature argument not name of a feature: %s', feature)
+            if warn:
+                _log.warn('rule Feature argument not name of a feature: %s', feature)
             self.feature = None
         self.feature = _F[feature]
 
@@ -566,11 +574,13 @@ class Feature(Condition):
 
 
 class Report(Condition):
-    def __init__(self, report):
+    def __init__(self, report, warn=True):
         if not (isinstance(report, int)):
-            _log.warn('rule Report argument not an integer: %s', report)
+            if warn:
+                _log.warn('rule Report argument not an integer: %s', report)
             self.report = -1
-        self.report = report
+        else:
+            self.report = report
 
     def __str__(self):
         return 'Report: ' + str(self.report)
@@ -584,9 +594,10 @@ class Report(Condition):
 
 # Setting(device, setting, [key], value...)
 class Setting(Condition):
-    def __init__(self, args):
+    def __init__(self, args, warn=True):
         if not (isinstance(args, list) and len(args) > 2):
-            _log.warn('rule Setting argument not list with minimum length 3: %s', args)
+            if warn:
+                _log.warn('rule Setting argument not list with minimum length 3: %s', args)
             self.args = []
         else:
             self.args = args
@@ -595,10 +606,9 @@ class Setting(Condition):
         return 'Setting: ' + ' '.join([str(a) for a in self.args])
 
     def evaluate(self, report, notification, device, status, last_result):
-        import solaar.ui.window as _window
         if len(self.args) < 3:
             return None
-        dev = _window.find_device(self.args[0]) if self.args[0] is not None else device
+        dev = _Device.find(self.args[0]) if self.args[0] is not None else device
         if dev is None:
             _log.warn('Setting condition: device %s is not known', self.args[0])
             return False
@@ -629,7 +639,7 @@ MODIFIER_MASK = MODIFIERS['Shift'] + MODIFIERS['Control'] + MODIFIERS['Alt'] + M
 
 
 class Modifiers(Condition):
-    def __init__(self, modifiers):
+    def __init__(self, modifiers, warn=True):
         modifiers = [modifiers] if isinstance(modifiers, str) else modifiers
         self.desired = 0
         self.modifiers = []
@@ -638,7 +648,8 @@ class Modifiers(Condition):
                 self.desired += MODIFIERS.get(k, 0)
                 self.modifiers.append(k)
             else:
-                _log.warn('unknown rule Modifier value: %s', k)
+                if warn:
+                    _log.warn('unknown rule Modifier value: %s', k)
 
     def __str__(self):
         return 'Modifiers: ' + str(self.desired)
@@ -659,14 +670,15 @@ class Key(Condition):
     DOWN = 'pressed'
     UP = 'released'
 
-    def __init__(self, args):
+    def __init__(self, args, warn=True):
         default_key = 0
         default_action = self.DOWN
 
         key, action = None, None
 
         if not args or not isinstance(args, (list, str)):
-            _log.warn('rule Key arguments unknown: %s' % args)
+            if warn:
+                _log.warn('rule Key arguments unknown: %s' % args)
             key = default_key
             action = default_action
         elif isinstance(args, str):
@@ -683,13 +695,15 @@ class Key(Condition):
         if isinstance(key, str) and key in _CONTROL:
             self.key = _CONTROL[key]
         else:
-            _log.warn('rule Key key name not name of a Logitech key: %s' % key)
+            if warn:
+                _log.warn('rule Key key name not name of a Logitech key: %s' % key)
             self.key = default_key
 
         if isinstance(action, str) and action in (self.DOWN, self.UP):
             self.action = action
         else:
-            _log.warn('rule Key action unknown: %s, assuming %s' % (action, default_action))
+            if warn:
+                _log.warn('rule Key action unknown: %s, assuming %s' % (action, default_action))
             self.action = default_action
 
     def __str__(self):
@@ -715,24 +729,28 @@ def range_test(start, end, min, max):
 
 
 class Test(Condition):
-    def __init__(self, test):
+    def __init__(self, test, warn=True):
         self.test = test
         if isinstance(test, str):
             if test in MOUSE_GESTURE_TESTS:
-                _log.warn('mouse movement test %s deprecated, converting to a MouseGesture', test)
+                if warn:
+                    _log.warn('mouse movement test %s deprecated, converting to a MouseGesture', test)
                 self.__class__ = MouseGesture
-                self.__init__(MOUSE_GESTURE_TESTS[test])
+                self.__init__(MOUSE_GESTURE_TESTS[test], warn=warn)
             elif test in TESTS:
                 self.function = TESTS[test]
             else:
-                _log.warn('rule Test string argument not name of a test: %s', test)
+                if warn:
+                    _log.warn('rule Test string argument not name of a test: %s', test)
                 self.function = TESTS['False']
         elif isinstance(test, list) and all(isinstance(t, int) for t in test):
-            _log.warn('Test rules consisting of numbers are deprecated, converting to a TestBytes condition')
+            if warn:
+                _log.warn('Test rules consisting of numbers are deprecated, converting to a TestBytes condition')
             self.__class__ = TestBytes
-            self.__init__(test)
+            self.__init__(test, warn=warn)
         else:
-            _log.warn('rule Test argument not valid %s', test)
+            if warn:
+                _log.warn('rule Test argument not valid %s', test)
 
     def __str__(self):
         return 'Test: ' + str(self.test)
@@ -745,7 +763,7 @@ class Test(Condition):
 
 
 class TestBytes(Condition):
-    def __init__(self, test):
+    def __init__(self, test, warn=True):
         self.test = test
         if (
             isinstance(test, list) and 2 < len(test) <= 4 and all(isinstance(t, int) for t in test) and test[0] >= 0
@@ -753,7 +771,8 @@ class TestBytes(Condition):
         ):
             self.function = bit_test(*test) if len(test) == 3 else range_test(*test)
         else:
-            _log.warn('rule TestBytes argument not valid %s', test)
+            if warn:
+                _log.warn('rule TestBytes argument not valid %s', test)
 
     def __str__(self):
         return 'TestBytes: ' + str(self.test)
@@ -771,12 +790,13 @@ class MouseGesture(Condition):
         'Mouse Down-right'
     ]
 
-    def __init__(self, movements):
+    def __init__(self, movements, warn=True):
         if isinstance(movements, str):
             movements = [movements]
         for x in movements:
             if x not in self.MOVEMENTS and x not in _CONTROL:
-                _log.warn('rule Key argument not name of a Logitech key: %s', x)
+                if warn:
+                    _log.warn('rule Mouse Gesture argument not direction or name of a Logitech key: %s', x)
         self.movements = movements
 
     def __str__(self):
@@ -785,32 +805,51 @@ class MouseGesture(Condition):
     def evaluate(self, feature, notification, device, status, last_result):
         if feature == _F.MOUSE_GESTURE:
             d = notification.data
-            count = _unpack('!h', d[:2])[0]
-            data = _unpack('!' + ((int(len(d) / 2) - 1) * 'h'), d[2:])
-            if count != len(self.movements):
-                return False
-            x = 0
-            z = 0
-            while x < len(data):
-                if data[x] == 0:
-                    direction = xy_direction(data[x + 1], data[x + 2])
-                    if self.movements[z] != direction:
+            data = _unpack('!' + (int(len(d) / 2) * 'h'), d)
+            data_offset = 1
+            movement_offset = 0
+            if self.movements and self.movements[0] not in self.MOVEMENTS:  # matching against initiating key
+                movement_offset = 1
+                if self.movements[0] != str(_CONTROL[data[0]]):
+                    return False
+            for m in self.movements[movement_offset:]:
+                if data_offset >= len(data):
+                    return False
+                if data[data_offset] == 0:
+                    direction = xy_direction(data[data_offset + 1], data[data_offset + 2])
+                    if m != direction:
                         return False
-                    x += 3
-                elif data[x] == 1:
-                    if data[x + 1] not in _CONTROL:
+                    data_offset += 3
+                elif data[data_offset] == 1:
+                    if m != str(_CONTROL[data[data_offset + 1]]):
                         return False
-                    if self.movements[z] != str(_CONTROL[data[x + 1]]):
-                        return False
-                    x += 2
-                z += 1
-            return True
+                    data_offset += 2
+            return data_offset == len(data)
         return False
 
     def data(self):
         return {'MouseGesture': [str(m) for m in self.movements]}
 
 
+class Active(Condition):
+    def __init__(self, devID, warn=True):
+        if not (isinstance(devID, str)):
+            if warn:
+                _log.warn('rule Active argument not a string: %s', devID)
+            self.devID = ''
+        self.devID = devID
+
+    def __str__(self):
+        return 'Active: ' + str(self.devID)
+
+    def evaluate(self, feature, notification, device, status, last_result):
+        dev = _Device.find(self.devID)
+        return bool(dev and dev.ping())
+
+    def data(self):
+        return {'Active': self.devID}
+
+
 class Action(RuleComponent):
     def __init__(self, *args):
         pass
@@ -822,15 +861,17 @@ class Action(RuleComponent):
 class KeyPress(Action):
     CLICK, DEPRESS, RELEASE = 'click', 'depress', 'release'
 
-    def __init__(self, args):
+    def __init__(self, args, warn=True):
         self.key_names, self.action = self.regularize_args(args)
         if not isinstance(self.key_names, list):
-            _log.warn('rule KeyPress keys not key names %s', self.keys_names)
+            if warn:
+                _log.warn('rule KeyPress keys not key names %s', self.keys_names)
             self.key_symbols = []
         else:
             self.key_symbols = [XK_KEYS.get(k, None) for k in self.key_names]
         if not all(self.key_symbols):
-            _log.warn('rule KeyPress keys not key names %s', self.key_names)
+            if warn:
+                _log.warn('rule KeyPress keys not key names %s', self.key_names)
             self.key_symbols = []
 
     def regularize_args(self, args):
@@ -923,12 +964,13 @@ class KeyPress(Action):
 
 
 class MouseScroll(Action):
-    def __init__(self, amounts):
+    def __init__(self, amounts, warn=True):
         import numbers
         if len(amounts) == 1 and isinstance(amounts[0], list):
             amounts = amounts[0]
         if not (len(amounts) == 2 and all([isinstance(a, numbers.Number) for a in amounts])):
-            _log.warn('rule MouseScroll argument not two numbers %s', amounts)
+            if warn:
+                _log.warn('rule MouseScroll argument not two numbers %s', amounts)
             amounts = [0, 0]
         self.amounts = amounts
 
@@ -953,20 +995,22 @@ class MouseScroll(Action):
 
 
 class MouseClick(Action):
-    def __init__(self, args):
+    def __init__(self, args, warn=True):
         if len(args) == 1 and isinstance(args[0], list):
             args = args[0]
         if not isinstance(args, list):
             args = [args]
         self.button = str(args[0]) if len(args) >= 0 else None
         if self.button not in buttons:
-            _log.warn('rule MouseClick action: button %s not known', self.button)
+            if warn:
+                _log.warn('rule MouseClick action: button %s not known', self.button)
             self.button = None
         count = args[1] if len(args) >= 2 else 1
         try:
             self.count = int(count)
         except (ValueError, TypeError):
-            _log.warn('rule MouseClick action: count %s should be an integer', count)
+            if warn:
+                _log.warn('rule MouseClick action: count %s should be an integer', count)
             self.count = 1
 
     def __str__(self):
@@ -985,9 +1029,10 @@ class MouseClick(Action):
 
 
 class Set(Action):
-    def __init__(self, args):
+    def __init__(self, args, warn=True):
         if not (isinstance(args, list) and len(args) > 2):
-            _log.warn('rule Set argument not list with minimum length 3: %s', args)
+            if warn:
+                _log.warn('rule Set argument not list with minimum length 3: %s', args)
             self.args = []
         else:
             self.args = args
@@ -996,24 +1041,24 @@ class Set(Action):
         return 'Set: ' + ' '.join([str(a) for a in self.args])
 
     def evaluate(self, feature, notification, device, status, last_result):
-        import solaar.ui.window as _window
         # importing here to avoid circular imports
+        from solaar.ui.config_panel import change_setting as _change_setting
 
         if len(self.args) < 3:
             return None
         if _log.isEnabledFor(_INFO):
             _log.info('Set action: %s', self.args)
-        dev = _window.find_device(self.args[0]) if self.args[0] is not None else device
+        dev = _Device.find(self.args[0]) if self.args[0] is not None else device
         if dev is None:
-            _log.error('Set action: device %s is not known', self.args[0])
+            _log.warn('Set action: device %s is not known', self.args[0])
             return None
         setting = next((s for s in dev.settings if s.name == self.args[1]), None)
         if setting is None:
-            _log.error('Set action: setting %s is not the name of a setting for %s', self.args[1], dev.name)
+            _log.warn('Set action: setting %s is not the name of a setting for %s', self.args[1], dev.name)
             return None
         args = setting.acceptable(self.args[2:], setting.read())
         if args is None:
-            _log.error('Set Action: invalid args %s for setting %s of %s', self.args[2:], self.args[1], self.args[0])
+            _log.warn('Set Action: invalid args %s for setting %s of %s', self.args[2:], self.args[1], self.args[0])
             return None
         _change_setting(dev, setting, args)
         return None
@@ -1023,11 +1068,12 @@ class Set(Action):
 
 
 class Execute(Action):
-    def __init__(self, args):
+    def __init__(self, args, warn=True):
         if isinstance(args, str):
             args = [args]
         if not (isinstance(args, list) and all(isinstance(arg), str) for arg in args):
-            _log.warn('rule Execute argument not list of strings: %s', args)
+            if warn:
+                _log.warn('rule Execute argument not list of strings: %s', args)
             self.args = []
         else:
             self.args = args
@@ -1061,6 +1107,7 @@ COMPONENTS = {
     'Test': Test,
     'TestBytes': TestBytes,
     'MouseGesture': MouseGesture,
+    'Active': Active,
     'KeyPress': KeyPress,
     'MouseScroll': MouseScroll,
     'MouseClick': MouseClick,
@@ -1185,7 +1232,7 @@ def _save_config_rule_file(file_name=_file_path):
     }
     # Save only user-defined rules
     rules_to_save = sum((r.data()['Rule'] for r in rules.components if r.source == file_name), [])
-    if rules_to_save:
+    if True:  # save even if there are no rules to save
         if _log.isEnabledFor(_INFO):
             _log.info('saving %d rule(s) to %s', len(rules_to_save), file_name)
         try:
